@@ -4,7 +4,7 @@ import os
 import pathlib
 import tempfile
 import warnings
-from typing import Any, Dict, List, Literal, Optional, TypedDict, Union, cast, overload
+from typing import Any, Literal, Optional, TypedDict, Union, cast, overload
 
 import yaml
 
@@ -47,6 +47,7 @@ class ServiceInfo(TypedDict):
 class ModelOperator:
     INFERENCE_SERVICE_ENDPOINT_NAME = "inference"
     INGRESS_ENDPOINT_URL_SUFFIX = "snowflakecomputing.app"
+    PRIVATELINK_INGRESS_ENDPOINT_URL_SUBSTRING = "privatelink.snowflakecomputing"
 
     def __init__(
         self,
@@ -104,7 +105,7 @@ class ModelOperator:
         *,
         database_name: Optional[sql_identifier.SqlIdentifier],
         schema_name: Optional[sql_identifier.SqlIdentifier],
-        statement_params: Optional[Dict[str, Any]] = None,
+        statement_params: Optional[dict[str, Any]] = None,
     ) -> str:
         stage_name = sql_identifier.SqlIdentifier(
             snowpark_utils.random_name_for_temp_object(snowpark_utils.TempObjectType.STAGE)
@@ -137,7 +138,7 @@ class ModelOperator:
         schema_name: Optional[sql_identifier.SqlIdentifier],
         model_name: sql_identifier.SqlIdentifier,
         version_name: sql_identifier.SqlIdentifier,
-        statement_params: Optional[Dict[str, Any]] = None,
+        statement_params: Optional[dict[str, Any]] = None,
     ) -> ModelAction:
         if self.validate_existence(
             database_name=database_name,
@@ -169,7 +170,7 @@ class ModelOperator:
         schema_name: Optional[sql_identifier.SqlIdentifier],
         model_name: sql_identifier.SqlIdentifier,
         version_name: sql_identifier.SqlIdentifier,
-        statement_params: Optional[Dict[str, Any]] = None,
+        statement_params: Optional[dict[str, Any]] = None,
     ) -> None:
         model_action = self.get_model_action_from_model_name_and_version(
             database_name=database_name,
@@ -205,7 +206,7 @@ class ModelOperator:
         schema_name: Optional[sql_identifier.SqlIdentifier],
         model_name: sql_identifier.SqlIdentifier,
         version_name: sql_identifier.SqlIdentifier,
-        statement_params: Optional[Dict[str, Any]] = None,
+        statement_params: Optional[dict[str, Any]] = None,
         use_live_commit: Optional[bool] = False,
     ) -> None:
 
@@ -263,7 +264,7 @@ class ModelOperator:
         model_name: sql_identifier.SqlIdentifier,
         version_name: sql_identifier.SqlIdentifier,
         model_exists: bool,
-        statement_params: Optional[Dict[str, Any]] = None,
+        statement_params: Optional[dict[str, Any]] = None,
     ) -> None:
         if model_exists:
             return self._model_version_client.add_version_from_model_version(
@@ -296,8 +297,8 @@ class ModelOperator:
         database_name: Optional[sql_identifier.SqlIdentifier],
         schema_name: Optional[sql_identifier.SqlIdentifier],
         model_name: Optional[sql_identifier.SqlIdentifier] = None,
-        statement_params: Optional[Dict[str, Any]] = None,
-    ) -> List[row.Row]:
+        statement_params: Optional[dict[str, Any]] = None,
+    ) -> list[row.Row]:
         if model_name:
             return self._model_client.show_versions(
                 database_name=database_name,
@@ -320,8 +321,8 @@ class ModelOperator:
         database_name: Optional[sql_identifier.SqlIdentifier],
         schema_name: Optional[sql_identifier.SqlIdentifier],
         model_name: Optional[sql_identifier.SqlIdentifier] = None,
-        statement_params: Optional[Dict[str, Any]] = None,
-    ) -> List[sql_identifier.SqlIdentifier]:
+        statement_params: Optional[dict[str, Any]] = None,
+    ) -> list[sql_identifier.SqlIdentifier]:
         res = self.show_models_or_versions(
             database_name=database_name,
             schema_name=schema_name,
@@ -341,7 +342,7 @@ class ModelOperator:
         schema_name: Optional[sql_identifier.SqlIdentifier],
         model_name: sql_identifier.SqlIdentifier,
         version_name: Optional[sql_identifier.SqlIdentifier] = None,
-        statement_params: Optional[Dict[str, Any]] = None,
+        statement_params: Optional[dict[str, Any]] = None,
     ) -> bool:
         if version_name:
             res = self._model_client.show_versions(
@@ -369,7 +370,7 @@ class ModelOperator:
         schema_name: Optional[sql_identifier.SqlIdentifier],
         model_name: sql_identifier.SqlIdentifier,
         version_name: Optional[sql_identifier.SqlIdentifier] = None,
-        statement_params: Optional[Dict[str, Any]] = None,
+        statement_params: Optional[dict[str, Any]] = None,
     ) -> str:
         if version_name:
             res = self._model_client.show_versions(
@@ -398,7 +399,7 @@ class ModelOperator:
         schema_name: Optional[sql_identifier.SqlIdentifier],
         model_name: sql_identifier.SqlIdentifier,
         version_name: Optional[sql_identifier.SqlIdentifier] = None,
-        statement_params: Optional[Dict[str, Any]] = None,
+        statement_params: Optional[dict[str, Any]] = None,
     ) -> None:
         if version_name:
             self._model_version_client.set_comment(
@@ -426,7 +427,7 @@ class ModelOperator:
         schema_name: Optional[sql_identifier.SqlIdentifier],
         model_name: sql_identifier.SqlIdentifier,
         version_name: sql_identifier.SqlIdentifier,
-        statement_params: Optional[Dict[str, Any]] = None,
+        statement_params: Optional[dict[str, Any]] = None,
     ) -> None:
         self._model_version_client.set_alias(
             alias_name=alias_name,
@@ -444,7 +445,7 @@ class ModelOperator:
         database_name: Optional[sql_identifier.SqlIdentifier],
         schema_name: Optional[sql_identifier.SqlIdentifier],
         model_name: sql_identifier.SqlIdentifier,
-        statement_params: Optional[Dict[str, Any]] = None,
+        statement_params: Optional[dict[str, Any]] = None,
     ) -> None:
         self._model_version_client.unset_alias(
             database_name=database_name,
@@ -461,7 +462,7 @@ class ModelOperator:
         schema_name: Optional[sql_identifier.SqlIdentifier],
         model_name: sql_identifier.SqlIdentifier,
         version_name: sql_identifier.SqlIdentifier,
-        statement_params: Optional[Dict[str, Any]] = None,
+        statement_params: Optional[dict[str, Any]] = None,
     ) -> None:
         if not self.validate_existence(
             database_name=database_name,
@@ -485,7 +486,7 @@ class ModelOperator:
         database_name: Optional[sql_identifier.SqlIdentifier],
         schema_name: Optional[sql_identifier.SqlIdentifier],
         model_name: sql_identifier.SqlIdentifier,
-        statement_params: Optional[Dict[str, Any]] = None,
+        statement_params: Optional[dict[str, Any]] = None,
     ) -> sql_identifier.SqlIdentifier:
         res = self._model_client.show_models(
             database_name=database_name,
@@ -504,7 +505,7 @@ class ModelOperator:
         schema_name: Optional[sql_identifier.SqlIdentifier],
         model_name: sql_identifier.SqlIdentifier,
         alias_name: sql_identifier.SqlIdentifier,
-        statement_params: Optional[Dict[str, Any]] = None,
+        statement_params: Optional[dict[str, Any]] = None,
     ) -> Optional[sql_identifier.SqlIdentifier]:
         res = self._model_client.show_versions(
             database_name=database_name,
@@ -528,7 +529,7 @@ class ModelOperator:
         tag_database_name: Optional[sql_identifier.SqlIdentifier],
         tag_schema_name: Optional[sql_identifier.SqlIdentifier],
         tag_name: sql_identifier.SqlIdentifier,
-        statement_params: Optional[Dict[str, Any]] = None,
+        statement_params: Optional[dict[str, Any]] = None,
     ) -> Optional[str]:
         r = self._tag_client.get_tag_value(
             database_name=database_name,
@@ -550,15 +551,15 @@ class ModelOperator:
         database_name: Optional[sql_identifier.SqlIdentifier],
         schema_name: Optional[sql_identifier.SqlIdentifier],
         model_name: sql_identifier.SqlIdentifier,
-        statement_params: Optional[Dict[str, Any]] = None,
-    ) -> Dict[str, str]:
+        statement_params: Optional[dict[str, Any]] = None,
+    ) -> dict[str, str]:
         tags_info = self._tag_client.get_tag_list(
             database_name=database_name,
             schema_name=schema_name,
             model_name=model_name,
             statement_params=statement_params,
         )
-        res: Dict[str, str] = {
+        res: dict[str, str] = {
             identifier.get_schema_level_object_identifier(
                 sql_identifier.SqlIdentifier(r.TAG_DATABASE, case_sensitive=True),
                 sql_identifier.SqlIdentifier(r.TAG_SCHEMA, case_sensitive=True),
@@ -578,7 +579,7 @@ class ModelOperator:
         tag_schema_name: Optional[sql_identifier.SqlIdentifier],
         tag_name: sql_identifier.SqlIdentifier,
         tag_value: str,
-        statement_params: Optional[Dict[str, Any]] = None,
+        statement_params: Optional[dict[str, Any]] = None,
     ) -> None:
         self._tag_client.set_tag_on_model(
             database_name=database_name,
@@ -600,7 +601,7 @@ class ModelOperator:
         tag_database_name: Optional[sql_identifier.SqlIdentifier],
         tag_schema_name: Optional[sql_identifier.SqlIdentifier],
         tag_name: sql_identifier.SqlIdentifier,
-        statement_params: Optional[Dict[str, Any]] = None,
+        statement_params: Optional[dict[str, Any]] = None,
     ) -> None:
         self._tag_client.unset_tag_on_model(
             database_name=database_name,
@@ -612,6 +613,30 @@ class ModelOperator:
             statement_params=statement_params,
         )
 
+    def _is_privatelink_connection(self) -> bool:
+        """Detect if the current session is using a privatelink connection."""
+        try:
+            host = self._session.connection.host
+            return ModelOperator.PRIVATELINK_INGRESS_ENDPOINT_URL_SUBSTRING in host
+        except AttributeError:
+            return False
+
+    def _extract_and_validate_ingress_url(self, res_row: "row.Row") -> Optional[str]:
+        """Extract and validate ingress URL from endpoint row."""
+        url_value = res_row[self._service_client.MODEL_INFERENCE_SERVICE_ENDPOINT_INGRESS_URL_COL_NAME]
+        if url_value is None:
+            return None
+        url_str = str(url_value)
+        return url_str if url_str.endswith(ModelOperator.INGRESS_ENDPOINT_URL_SUFFIX) else None
+
+    def _extract_and_validate_privatelink_url(self, res_row: "row.Row") -> Optional[str]:
+        """Extract and validate privatelink ingress URL from endpoint row."""
+        url_value = res_row[self._service_client.MODEL_INFERENCE_SERVICE_ENDPOINT_PRIVATELINK_INGRESS_URL_COL_NAME]
+        if url_value is None:
+            return None
+        url_str = str(url_value)
+        return url_str if ModelOperator.PRIVATELINK_INGRESS_ENDPOINT_URL_SUBSTRING in url_str else None
+
     def show_services(
         self,
         *,
@@ -619,8 +644,8 @@ class ModelOperator:
         schema_name: Optional[sql_identifier.SqlIdentifier],
         model_name: sql_identifier.SqlIdentifier,
         version_name: sql_identifier.SqlIdentifier,
-        statement_params: Optional[Dict[str, Any]] = None,
-    ) -> List[ServiceInfo]:
+        statement_params: Optional[dict[str, Any]] = None,
+    ) -> list[ServiceInfo]:
         res = self._model_client.show_versions(
             database_name=database_name,
             schema_name=schema_name,
@@ -643,30 +668,41 @@ class ModelOperator:
         # TODO(sdas): Figure out a better way to filter out MODEL_BUILD_ services server side.
         fully_qualified_service_names = [str(service) for service in json_array if "MODEL_BUILD_" not in service]
 
-        result = []
+        result: list[ServiceInfo] = []
+        is_privatelink_connection = self._is_privatelink_connection()
 
         for fully_qualified_service_name in fully_qualified_service_names:
-            ingress_url: Optional[str] = None
+            inference_endpoint: Optional[str] = None
             db, schema, service_name = sql_identifier.parse_fully_qualified_name(fully_qualified_service_name)
-            service_status, _ = self._service_client.get_service_status(
+            statuses = self._service_client.get_service_container_statuses(
                 database_name=db, schema_name=schema, service_name=service_name, statement_params=statement_params
             )
+            if len(statuses) == 0:
+                return result
+
+            service_status = statuses[0].service_status
             for res_row in self._service_client.show_endpoints(
                 database_name=db, schema_name=schema, service_name=service_name, statement_params=statement_params
             ):
                 if (
                     res_row[self._service_client.MODEL_INFERENCE_SERVICE_ENDPOINT_NAME_COL_NAME]
-                    == self.INFERENCE_SERVICE_ENDPOINT_NAME
-                    and res_row[self._service_client.MODEL_INFERENCE_SERVICE_ENDPOINT_INGRESS_URL_COL_NAME] is not None
+                    != self.INFERENCE_SERVICE_ENDPOINT_NAME
                 ):
-                    ingress_url = str(
-                        res_row[self._service_client.MODEL_INFERENCE_SERVICE_ENDPOINT_INGRESS_URL_COL_NAME]
-                    )
-                    if not ingress_url.endswith(ModelOperator.INGRESS_ENDPOINT_URL_SUFFIX):
-                        ingress_url = None
+                    continue
+
+                ingress_url = self._extract_and_validate_ingress_url(res_row)
+                privatelink_ingress_url = self._extract_and_validate_privatelink_url(res_row)
+
+                if is_privatelink_connection and privatelink_ingress_url is not None:
+                    inference_endpoint = privatelink_ingress_url
+                else:
+                    inference_endpoint = ingress_url
+
             result.append(
                 ServiceInfo(
-                    name=fully_qualified_service_name, status=service_status.value, inference_endpoint=ingress_url
+                    name=fully_qualified_service_name,
+                    status=service_status.value,
+                    inference_endpoint=inference_endpoint,
                 )
             )
 
@@ -682,7 +718,7 @@ class ModelOperator:
         service_database_name: Optional[sql_identifier.SqlIdentifier],
         service_schema_name: Optional[sql_identifier.SqlIdentifier],
         service_name: sql_identifier.SqlIdentifier,
-        statement_params: Optional[Dict[str, Any]] = None,
+        statement_params: Optional[dict[str, Any]] = None,
     ) -> None:
         services = self.show_services(
             database_name=database_name,
@@ -724,7 +760,7 @@ class ModelOperator:
         schema_name: Optional[sql_identifier.SqlIdentifier],
         model_name: sql_identifier.SqlIdentifier,
         version_name: sql_identifier.SqlIdentifier,
-        statement_params: Optional[Dict[str, Any]] = None,
+        statement_params: Optional[dict[str, Any]] = None,
     ) -> model_manifest_schema.ModelManifestDict:
         with tempfile.TemporaryDirectory() as tmpdir:
             self._model_version_client.get_file(
@@ -741,9 +777,9 @@ class ModelOperator:
 
     @staticmethod
     def _match_model_spec_with_sql_functions(
-        sql_functions_names: List[sql_identifier.SqlIdentifier], target_methods: List[str]
-    ) -> Dict[sql_identifier.SqlIdentifier, str]:
-        res: Dict[sql_identifier.SqlIdentifier, str] = {}
+        sql_functions_names: list[sql_identifier.SqlIdentifier], target_methods: list[str]
+    ) -> dict[sql_identifier.SqlIdentifier, str]:
+        res: dict[sql_identifier.SqlIdentifier, str] = {}
 
         for target_method in target_methods:
             # Here we need to find the SQL function corresponding to the Python function.
@@ -766,7 +802,7 @@ class ModelOperator:
         schema_name: Optional[sql_identifier.SqlIdentifier],
         model_name: sql_identifier.SqlIdentifier,
         version_name: sql_identifier.SqlIdentifier,
-        statement_params: Optional[Dict[str, Any]] = None,
+        statement_params: Optional[dict[str, Any]] = None,
     ) -> model_meta_schema.ModelMetadataDict:
         raw_model_spec_res = self._model_client.show_versions(
             database_name=database_name,
@@ -787,7 +823,7 @@ class ModelOperator:
         schema_name: Optional[sql_identifier.SqlIdentifier],
         model_name: sql_identifier.SqlIdentifier,
         version_name: sql_identifier.SqlIdentifier,
-        statement_params: Optional[Dict[str, Any]] = None,
+        statement_params: Optional[dict[str, Any]] = None,
     ) -> type_hints.Task:
         model_version = self._model_client.show_versions(
             database_name=database_name,
@@ -809,8 +845,8 @@ class ModelOperator:
         schema_name: Optional[sql_identifier.SqlIdentifier],
         model_name: sql_identifier.SqlIdentifier,
         version_name: sql_identifier.SqlIdentifier,
-        statement_params: Optional[Dict[str, Any]] = None,
-    ) -> List[model_manifest_schema.ModelFunctionInfo]:
+        statement_params: Optional[dict[str, Any]] = None,
+    ) -> list[model_manifest_schema.ModelFunctionInfo]:
         model_spec = self._fetch_model_spec(
             database_name=database_name,
             schema_name=schema_name,
@@ -907,7 +943,7 @@ class ModelOperator:
         version_name: sql_identifier.SqlIdentifier,
         strict_input_validation: bool = False,
         partition_column: Optional[sql_identifier.SqlIdentifier] = None,
-        statement_params: Optional[Dict[str, str]] = None,
+        statement_params: Optional[dict[str, str]] = None,
         is_partitioned: Optional[bool] = None,
     ) -> Union[type_hints.SupportedDataType, dataframe.DataFrame]:
         ...
@@ -923,7 +959,7 @@ class ModelOperator:
         schema_name: Optional[sql_identifier.SqlIdentifier],
         service_name: sql_identifier.SqlIdentifier,
         strict_input_validation: bool = False,
-        statement_params: Optional[Dict[str, str]] = None,
+        statement_params: Optional[dict[str, str]] = None,
     ) -> Union[type_hints.SupportedDataType, dataframe.DataFrame]:
         ...
 
@@ -941,7 +977,7 @@ class ModelOperator:
         service_name: Optional[sql_identifier.SqlIdentifier] = None,
         strict_input_validation: bool = False,
         partition_column: Optional[sql_identifier.SqlIdentifier] = None,
-        statement_params: Optional[Dict[str, str]] = None,
+        statement_params: Optional[dict[str, str]] = None,
         is_partitioned: Optional[bool] = None,
     ) -> Union[type_hints.SupportedDataType, dataframe.DataFrame]:
         identifier_rule = model_signature.SnowparkIdentifierRule.INFERRED
@@ -952,7 +988,7 @@ class ModelOperator:
             output_with_input_features = False
             df = model_signature._convert_and_validate_local_data(X, signature.inputs, strict=strict_input_validation)
             s_df = snowpark_handler.SnowparkDataFrameHandler.convert_from_df(
-                self._session, df, keep_order=keep_order, features=signature.inputs
+                self._session, df, keep_order=keep_order, features=signature.inputs, statement_params=statement_params
             )
         else:
             keep_order = False
@@ -966,9 +1002,16 @@ class ModelOperator:
 
         # Compose input and output names
         input_args = []
+        quoted_identifiers_ignore_case = (
+            snowpark_handler.SnowparkDataFrameHandler._is_quoted_identifiers_ignore_case_enabled(
+                self._session, statement_params
+            )
+        )
+
         for input_feature in signature.inputs:
             col_name = identifier_rule.get_sql_identifier_from_feature(input_feature.name)
-
+            if quoted_identifiers_ignore_case:
+                col_name = sql_identifier.SqlIdentifier(input_feature.name.upper(), case_sensitive=True)
             input_args.append(col_name)
 
         returns = []
@@ -1048,7 +1091,9 @@ class ModelOperator:
 
         # Get final result
         if not isinstance(X, dataframe.DataFrame):
-            return snowpark_handler.SnowparkDataFrameHandler.convert_to_df(df_res, features=signature.outputs)
+            return snowpark_handler.SnowparkDataFrameHandler.convert_to_df(
+                df_res, features=signature.outputs, statement_params=statement_params
+            )
         else:
             return df_res
 
@@ -1059,7 +1104,7 @@ class ModelOperator:
         schema_name: Optional[sql_identifier.SqlIdentifier],
         model_name: sql_identifier.SqlIdentifier,
         version_name: Optional[sql_identifier.SqlIdentifier] = None,
-        statement_params: Optional[Dict[str, Any]] = None,
+        statement_params: Optional[dict[str, Any]] = None,
     ) -> None:
         if version_name:
             self._model_version_client.drop_version(
@@ -1086,7 +1131,7 @@ class ModelOperator:
         new_model_db: Optional[sql_identifier.SqlIdentifier],
         new_model_schema: Optional[sql_identifier.SqlIdentifier],
         new_model_name: sql_identifier.SqlIdentifier,
-        statement_params: Optional[Dict[str, Any]] = None,
+        statement_params: Optional[dict[str, Any]] = None,
     ) -> None:
         self._model_client.rename(
             database_name=database_name,
@@ -1121,7 +1166,7 @@ class ModelOperator:
         version_name: sql_identifier.SqlIdentifier,
         target_path: pathlib.Path,
         mode: Literal["full", "model", "minimal"] = "model",
-        statement_params: Optional[Dict[str, Any]] = None,
+        statement_params: Optional[dict[str, Any]] = None,
     ) -> None:
         for remote_rel_path, is_dir in self.MODEL_FILE_DOWNLOAD_PATTERN[mode].items():
             list_file_res = self._model_version_client.list_file(
